''' <summary>
''' Program to test Serial Port Communication, structured error handling, and XML comments
''' </summary>
''' <remarks></remarks>
Public Class Form1

    Private WithEvents _serialPort As New IO.Ports.SerialPort

    ''' <summary>
    ''' Event handler for form load. Adds serial port names to the serial port list.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        Try
            For Each portName As String In My.Computer.Ports.SerialPortNames
                uxSerialPortList.Items.Add(portName)
            Next
            uxSerialPortList.SelectedIndex = 0
        Catch ex As Exception
            HandleException(ex)
        End Try

    End Sub


    ''' <summary>
    ''' Event handler for Connect Button click
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub uxConnectButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles uxConnectButton.Click

        Try
            With _serialPort
                .PortName = uxSerialPortList.Text
                .BaudRate = 115200
                .Parity = IO.Ports.Parity.None
                .DataBits = 8
                .StopBits = IO.Ports.StopBits.One
                .Handshake = IO.Ports.Handshake.RequestToSendXOnXOff
            End With
            _serialPort.Open()
            _serialPort.DtrEnable = True

            uxDataReceived.AppendText(String.Format("*** Connected to {0} ***" & vbCrLf, _serialPort.PortName))

        Catch ex As Exception
            HandleException(ex)
        End Try

    End Sub


    ''' <summary>
    ''' Event handler for Disconnect Button click
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub uxDisconnectButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles uxDisconnectButton.Click

        Try
            _serialPort.Close()
            uxDataReceived.AppendText(String.Format("*** Disconnected from {0} ***" & vbCrLf, _serialPort.PortName))
            _serialPort.Dispose()
        Catch ex As Exception
            HandleException(ex)
        End Try

    End Sub


    ''' <summary>
    ''' Event handler for Send Button click
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub uxSendButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles uxSendButton.Click

        SendData()

    End Sub


    ''' <summary>
    ''' Event handle for data textbox key press, used to trap carriage return to send data
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub uxDataToSend_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles uxDataToSend.KeyPress

        If e.KeyChar = vbCr Then SendData()

    End Sub


    ''' <summary>
    ''' Sends data from the data textbox and clears the textbox
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SendData()

        Try
            _serialPort.Write(uxDataToSend.Text + vbCr)
            uxDataToSend.Clear()
        Catch ex As Exception
            HandleException(ex)
        End Try

    End Sub


    ''' <summary>
    ''' Handles the event raised when the serial port receives data
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' Runs on a separate thread, requiring the use of a delegate to update the form
    ''' </remarks>
    Private Sub DataReceived(ByVal sender As Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles _serialPort.DataReceived

        uxDataReceived.Invoke(New UpdateDataReceivedDelegate(AddressOf UpdateDataReceived), New Object() {})

    End Sub


    ''' <summary>
    ''' Delegate for UpdateDataReceived
    ''' </summary>
    ''' <remarks></remarks>
    Private Delegate Sub UpdateDataReceivedDelegate()


    ''' <summary>
    ''' Read data from the serial port and add to the Data Received textbox
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub UpdateDataReceived()

        Dim s As String = _serialPort.ReadExisting
        s = s.Replace(vbCrLf, "<CR><LF>")
        s = s.Replace(vbCr, "<CR>" & vbCrLf)
        s = s.Replace("<CR><LF>", "<CR><LF>" & vbCrLf)

        uxDataReceived.AppendText(s)

    End Sub


    ''' <summary>
    ''' Used to handle any exceptions
    ''' </summary>
    ''' <param name="ex">Exception to handle</param>
    ''' <remarks></remarks>
    Private Sub HandleException(ByVal ex As Exception)
        'MessageBox.Show(ex.ToString, ex.Message)
        uxDataReceived.AppendText(ex.Message & vbCrLf)
    End Sub


    ''' <summary>
    ''' Subroutine to test help generation from XML comments
    ''' </summary>
    ''' <param name="s">A string goes here</param>
    ''' <param name="i">Put an integer here</param>
    ''' <remarks>Remarks</remarks>
    Public Sub TestSub(ByVal s As String, ByVal i As Integer)

    End Sub

    Private Sub uxSendWithoutCR_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles uxSendWithoutCR.Click
        Try
            _serialPort.Write(uxDataToSend.Text)
            uxDataToSend.Clear()
        Catch ex As Exception
            HandleException(ex)
        End Try
    End Sub

    Private Sub uxSendCR_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles uxSendCR.Click
        Try
            _serialPort.Write(vbCr)
        Catch ex As Exception
            HandleException(ex)
        End Try
    End Sub

    Private Sub uxSendCtrlZ_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles uxSendCtrlZ.Click
        Try
            _serialPort.Write(Chr(26))
        Catch ex As Exception
            HandleException(ex)
        End Try
    End Sub

End Class
